==> Ambidexterity-0.1.0%2B1.21.%289%2B%29-Fabric-sources.jar: eva/ambidexterity/mixin/ServerPlayerMixin.java <==
package eva.ambidexterity.mixin;

import com.llamalad7.mixinextras.sugar.Local;
import eva.ambidexterity.access.LivingEntityAccess;
import eva.ambidexterity.access.ServerPlayerAccess;
import net.minecraft.class_1268;
import net.minecraft.class_1297;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_1839;
import net.minecraft.class_3222;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.Redirect;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

@Mixin(class_3222.class)
public class ServerPlayerMixin extends PlayerMixin implements ServerPlayerAccess {

    @Inject(
            method = "swing",
            at = @At(
                    value = "INVOKE",
                    target = "Lnet/minecraft/server/level/ServerPlayer;resetAttackStrengthTicker()V",
                    shift = At.Shift.AFTER
            ),
            cancellable = true)
    private void swingHandMixin(class_1268 hand, CallbackInfo ci) {
        class_1799 itemStack = ((class_1657) (Object) this).method_6079();
        if (hand == class_1268.field_5810 && itemStack.method_7976() == class_1839.field_8952) {
            ci.cancel();
        }
    }

    @Override
    public void dualWielding$attackOffhand(class_1297 target) {
        super.dualWielding$attackOffhand(target);
    }

    @Redirect(
            method = "swing",
            at = @At(
                    value = "INVOKE",
                    target = "Lnet/minecraft/server/level/ServerPlayer;resetAttackStrengthTicker()V"
            )
    )
    private void reSwingable(class_3222 player, @Local(argsOnly = true) class_1268 hand) {
        boolean valHolder = ((LivingEntityAccess) player).offhand(hand == class_1268.field_5810);
        player.method_7350();
        ((LivingEntityAccess) player).offhand(valHolder);
    }

}

==> Ambidexterity-0.2.0%2B1.21.%289%2B%29-Fabric-sources.jar: eva/ambidexterity/mixin/ServerPlayerMixin.java <==
package eva.ambidexterity.mixin;

import com.llamalad7.mixinextras.sugar.Local;
import eva.ambidexterity.access.LivingEntityAccess;
import eva.ambidexterity.access.ServerPlayerAccess;
import net.minecraft.class_1268;
import net.minecraft.class_1297;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_1839;
import net.minecraft.class_3222;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.Redirect;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

@Mixin(class_3222.class)
public abstract class ServerPlayerMixin extends PlayerMixin implements ServerPlayerAccess {

    @Unique
    private boolean offhand = false;

    @Inject(
            method = "swing",
            at = @At(
                    value = "INVOKE",
                    target = "Lnet/minecraft/server/level/ServerPlayer;resetAttackStrengthTicker()V",
                    shift = At.Shift.AFTER
            ),
            cancellable = true)
    private void swingHandMixin(class_1268 hand, CallbackInfo ci) {
        class_1799 itemStack = ((class_1657) (Object) this).method_6079();
        if (hand == class_1268.field_5810 && itemStack.method_7976() == class_1839.field_8952) {
            ci.cancel();
        }
    }

    @Override
    public void dualWielding$attackOffhand(class_1297 target) {
        super.dualWielding$attackOffhand(target);
    }

    @Redirect(
            method = "swing",
            at = @At(
                    value = "INVOKE",
                    target = "Lnet/minecraft/server/level/ServerPlayer;resetAttackStrengthTicker()V"
            )
    )
    private void reSwingable(class_3222 player, @Local(argsOnly = true) class_1268 hand) {
        boolean valHolder = ((LivingEntityAccess) player).offhand(hand == class_1268.field_5810);
        player.method_7350();
        ((LivingEntityAccess) player).offhand(valHolder);
    }

    @Unique @Override
    public boolean isOffhand() {
        return offhand;
    }

    @Unique @Override
    public void setOffhand(boolean offhand) {
        this.offhand = offhand;
    }
}
